% bernoulli.m  Use FFT to compute Bernoulli numbers
%              L. N. Trefethen 2/05
%
% The nth Bernoulli number is n! times the nth Taylor coefficient of
% z/(exp(z)-1).  This code, a variant of taylor_coeffs.m, computes
% B_0,...,B_20 by the FFT.

  f = inline('z./(exp(z)-1)');           % generating function
  N = 128;                               % no. of sample points
  r = 4;                                 % radius of sample circle
  zz = r*exp(2i*pi*(0:N-1)/N);           % sample points
  c = real(fft(f(zz)))/N;                % Taylor coeffs. via FFT
  c = c.*gamma(1:N)./r.^(0:N-1);         % rescale            
  
  exact = [1 -1/2 1/6 0 -1/30 0 1/42 0 -1/30 0 5/66 0 -691/2730 ...
           0 7/6 0 -3617/510 0 43867/798 0 -174611/330];
  disp('  n         computed           exact         error')
  for n = 1:length(exact)
    fprintf('%3d %17.10f %17.10f %11.1e\n',...
            n-1,c(n),exact(n),c(n)-exact(n)')
  end
