% operator_norm.m  Norm of infinite matrix from "100 Digit-Challenge"   
%                  C. Ortner and L. N. Trefethen 2/05
%
% Let A be the infinite matrix with a_11=1, a_12=1/2, a_21=1/3,
% a_13=1/4, a_22=1/5, etc.  Problem 3 of the 100-Digit Challenge is
% to find the 2-norm of this operator; see Chap. 3 of Bornemann, et
% al., The SIAM 100-Digit Challenge, 2004.  This code gets the answer
% to about 12 digits by computing norms of matrices of dimensions
% 1,2,4,...,1024 and using epsilon extrapolation.

% Compute norms of finite matrices:
  M = 10; N = 2^M;                              % max matrix dimension
  A = zeros(N); b = cumsum(1:2*N)';
  for j = 1:N, A(:,j) = b(j+(1:N)-1)+1-j; end
  A = 1./A;                                     % matrix of that dim.
  data = [];
  for N = 2.^(0:M)
    data = [data; normest(A(1:N,1:N),5e-16)];   % norms of submatrices
  end
  format long
  disp('Raw data:'), disp(data)                 % print the data

% Extrapolate the data by epsilon algorithm:
  nd = length(data);
  E = zeros(nd,nd+1);
  E(:,2) = data;
  for j = 2:nd
    k = j:nd;                                   % epsilon extrap.
    E(k,j+1) = E(k-1,j-1)-1./(E(k,j)-E(k-1,j));
  end
  Y = diag(E,1); Y = Y(1:2:end);
  disp('Extrapolated values:'), disp(Y)
  disp('Exact solution:')
  disp('   1.274224152821228188212340...')
