% pi_via_agm.m  Compute pi by arithmetic-geometric mean iteration
%               L. N. Trefethen 7/03
%
% The AGM iteration for computing elliptic integrals goes back to
% Gauss.  In 1976 Brent (J. ACM) and independently Salamin (Math.
% Comp.) published AGM algorithms converging quadratically to pi.
% This code implements a variant due to Borwein and Borwein (SIAM
% Review 1984) given as Algorithm 2.1 in their 1987 book "Pi and
% the AGM".  The results are more dramatic in extended precision!

  y = sqrt(sqrt(2));
  x = (y+1/y)/2;
  p = 2+sqrt(2);
  for i = 1:6
    fprintf('%21.16g\n',p)
    p = p*(1+x)/(1+y);
    s = sqrt(x);
    y = (y*s+1/s)/(1+y);
    x = (s+1/s)/2;
  end
