% richardson.m  Richardson extrapolation for Archimedes' polygons
%               L. N. Trefethen 2/05
%
% This little code applies Richardson extrapolation to Archimedes'
% classic problem of approximating pi via perimeters of regular
% polygons inscribed in a circle.

  format long, format loose
  k = 2.^(1:5)';                               % geometrically varying k 
  a = k.*sin(pi./k);                           % data
  N = length(a);                               % number of data points
  A = NaN*ones(N); A(:,1) = a;                 % put data in matrix A
  for j = 1:N-1
    r = 4^(-j); f = (r/(1-r));                 % extrapolation factors
    a = a(2:end) + f*(a(2:end)-a(1:end-1));    % extrapolate
    A(1:N-j,j+1) = a;                          % store results in A
  end
  A
  error = pi-A
