
% euler_const.m  Compute Euler's constant by contour integral
%                Nick Trefethen 6/05
%
% It is known that euler = -Gamma'(1), where Gamma is the gamma
% function.  By Hankel's contour integral for Gamma this implies
%
% euler = (2*pi*i)^(-1) int_C e^t log(t)/t dt
%
% where the contour C passes from -infty-0i around 0 to -infty+0i.
% This code, adapted from gamma_talbot.m, computes the integral to
% 15-digit precision by adding up values from 15 sample points
% along a Talbot/Weideman contour.

  format long
  N = 30;                                        % twice the no. of pts
  th = pi*(1:2:N-1)/N;                           % angles in [0,pi]
  a = -.2407; b = .2387; c = .7409; d = .1349i;  % Weideman's parameters
  z = N*(a + b*th.*cot(c*th) + d*th);            % Talbot pts in z-plane
  zp = b*cot(c*th) - b*c*th./sin(c*th).^2 + d;   % N^(-1) times derivative
  euler = sum(zp.*exp(z).*log(z)./z);            % add values at sample pts
  euler = -2*imag(euler)                         % exploit symmetry
  exact = 0.57721566490153286060651209           % actual Euler constant
